package org.greenstone.gatherer.gui;
/**
 *#########################################################################
 *
 * A component of the Gatherer application, part of the Greenstone digital
 * library suite from the New Zealand Digital Library Project at the
 * University of Waikato, New Zealand.
 *
 * <BR><BR>
 *
 * Author: John Thompson, Greenstone Digital Library, University of Waikato
 *
 * <BR><BR>
 *
 * Copyright (C) 1999 New Zealand Digital Library Project
 *
 * <BR><BR>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * <BR><BR>
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * <BR><BR>
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *########################################################################
 */
import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;
import javax.swing.border.*;
import org.greenstone.gatherer.Gatherer;
/**
 * @author John Thompson, Greenstone Digital Library, University of Waikato
 * @version 2.3
 */
public class OptionsBox
    extends JDialog
    implements ActionListener, FocusListener {

    private JButton cancel_button;
    private JButton establish_connection;
    private JButton ok_button;

    private JCheckBox use_colours;
    private JCheckBox use_proxy;

    private JLabel proxy_host_label;
    private JLabel proxy_port_label;

    private JTabbedPane tab_pane;

    private JTextField proxy_host;
    private JTextField proxy_port;

    private JPanel button_pane;
    private JPanel connection_pane;
    private JPanel content_pane;
    private JPanel download_pane;
    private JPanel general_pane;
    private JPanel proxy_pane;
    private JPanel proxy_host_pane;
    private JPanel proxy_port_pane;

    static final private Dimension SIZE = new Dimension(400,300);

    public OptionsBox() {
	super(Gatherer.g_man);

	this.setModal(true);
	this.setTitle(get("Title"));
	this.setSize(SIZE);

	content_pane = (JPanel) this.getContentPane();
	content_pane.setLayout(new BorderLayout());
	content_pane.setMinimumSize(this.getSize());
	content_pane.setSize(this.getSize());
	content_pane.setMaximumSize(this.getSize());

	tab_pane = new JTabbedPane();

	tab_pane.add(get("General"), buildGeneral());
	if(!Gatherer.config.get("workflow.mirror", true)) {
	    tab_pane.add(get("Connection"), buildConnection());
	    tab_pane.add(get("Download"), buildDownload());
	}

	button_pane = new JPanel(new GridLayout(1,2));
	button_pane.setBorder(BorderFactory.createEmptyBorder(5,5,5,5));

	cancel_button = new JButton(get("General.Cancel"));
	cancel_button.addActionListener(this);

	ok_button = new JButton(get("General.OK"));
	ok_button.addActionListener(this);

	// Add components.
	content_pane.add(tab_pane, BorderLayout.CENTER);
	button_pane.add(ok_button);
	button_pane.add(cancel_button);
	content_pane.add(button_pane, BorderLayout.SOUTH);

	// Final dialog setup & positioning.
	// Position the window.
	Dimension screen_size = Gatherer.config.screen_size;
	setLocation((screen_size.width - SIZE.width) / 2, (screen_size.height - SIZE.height) / 2);
	show();
    }

    public void actionPerformed(ActionEvent event) {
	Object esrc = event.getSource();
	// OK - Save all settings.
	if(esrc == ok_button) {
				// Update config
	    this.dispose();
	}
	// Cancel - Ignore settings and close.
	else if(esrc == cancel_button) {
	    this.dispose();
	}
	else if(esrc == use_proxy) {
	    Gatherer.config.set("general.use_proxy", true, use_proxy.isSelected());
	    proxy_host.setEnabled(use_proxy.isSelected());
	    proxy_port.setEnabled(use_proxy.isSelected());
	    establish_connection.setEnabled(use_proxy.isSelected());
	}
	else if(esrc == establish_connection) {
	    if(use_proxy.isSelected()) {
		Gatherer.config.setString("general.proxy_host", true, proxy_host.getText());
		Gatherer.config.setString("general.proxy_port", true, proxy_port.getText());
		Gatherer.self.setProxy();
	    }
	}
    }

    public JPanel buildGeneral() {
	general_pane = new JPanel(new GridLayout(2,1));
	JPanel color_pane = new JPanel();
	use_colours = new JCheckBox(get("Use_Colours"));
	// Connection
	use_colours.addActionListener(this);
	// Layout
	color_pane.add(use_colours);

	general_pane.add(color_pane);
	return general_pane;
    }

    public JPanel buildConnection() {
	connection_pane = new JPanel(new BorderLayout());

	proxy_pane = new JPanel(new GridLayout(4,1));
	proxy_pane.setBorder(BorderFactory.createTitledBorder(
							      BorderFactory.createBevelBorder(BevelBorder.LOWERED), 
							      get("Proxy_Configuration")));

	use_proxy = new JCheckBox(get("Use_Proxy"));
	use_proxy.addActionListener(this);

	establish_connection = new JButton(get("Establish_Connection"));
	establish_connection.addActionListener(this);

	proxy_host_pane = new JPanel(new BorderLayout());
	proxy_host_label = new JLabel(get("Proxy_Host"));

	proxy_port_pane = new JPanel(new BorderLayout());
	proxy_port_label = new JLabel(get("Proxy_Port"));

	if(Gatherer.config.get("general.use_proxy", true)) {
	    use_proxy.setSelected(true);

	    proxy_host = new JTextField(Gatherer.config.getString("general.proxy_host", true));
	    proxy_port = new JTextField(Gatherer.config.getString("general.proxy_port", true));
	}
	else {
	    use_proxy.setSelected(false);
	    proxy_host = new JTextField();
	    proxy_host.setEnabled(false);
	    proxy_port = new JTextField();
	    proxy_port.setEnabled(false);

	    establish_connection.setEnabled(false);
	}

	proxy_host.setColumns(40);

	proxy_host.addActionListener(this);
	proxy_host.addFocusListener(this);
	proxy_port.addActionListener(this);
	proxy_port.addFocusListener(this);

	// Add lowest layer components
	proxy_host_pane.add(proxy_host_label, BorderLayout.WEST);
	proxy_host_pane.add(proxy_host, BorderLayout.CENTER);

	proxy_port_pane.add(proxy_port_label, BorderLayout.WEST);
	proxy_port_pane.add(proxy_port, BorderLayout.CENTER);

	// Add components
	proxy_pane.add(use_proxy);
	proxy_pane.add(proxy_host_pane);
	proxy_pane.add(proxy_port_pane);
	proxy_pane.add(establish_connection);

	// Add to higher layer.
	connection_pane.add(proxy_pane, BorderLayout.CENTER);

	return connection_pane;
    }

    public JPanel buildDownload() {
	download_pane = new JPanel(new BorderLayout());
	return download_pane;
    }

    public void destroy() {
	cancel_button = null;
	establish_connection = null;
	ok_button = null;
	use_colours = null;
	use_proxy = null;
	proxy_host_label = null;
	proxy_port_label = null;
	tab_pane = null;
	proxy_host = null;
	proxy_port = null;
	button_pane = null;
	connection_pane = null;
	content_pane = null;
	download_pane = null;
	general_pane = null;
	proxy_pane = null;
	proxy_host_pane = null;
	proxy_port_pane = null;	  
    }

    // We have to trap focus changes and change them into action events.
    public void focusGained(FocusEvent e) {
    }
	 
    public void focusLost(FocusEvent event) {
	actionPerformed(new ActionEvent(event.getSource(), event.getID(), ""));
    }
	 
    private String get(String key) {
	if(key.indexOf(".") == -1) {
	    key = "Options." + key;
	}
	return Gatherer.dictionary.get(key);
    }
}
